% This m-script performs Monte Carlo simulations for tests of multiple
% equilibria (Table 1) and signs of interaction effects (Table 2) in de 
% Paula and Tang (2011).  The computations are stored in the workspace
% "MCP1_jobnum.mat" where jobnum is an argument in the function MCP1.

% When the workspace is opened in Matlab, the results are obtained from
% the matrices DecMat1, DecMat2, DecMat3 and DecSgn. These matrices are
% S by 3.  A row corresponds to a Monte Carlo simulation and
% each column corresponds to a player.
% Table 1: RP1 is the first component of mean(DecMat1).
% Table 1: RP2 is the first component of mean(DecMat2).
% Table 1: RP3 is the first component of mean(DecMat3).
% Table 2: q+ is mean((DecSgn>0)) and q- is mean((DecSgn<0)).

function MCP1(G,B,alpha,Lambda,S,jobnum);
% G is the sample size in each simulation.
% B is the number of bootstrap samples for Romano and Wolf (2005).
% alpha is the desired FWER.
% Lambda is the probability that equilibrium (p1,p2,p3)=
%               (0.0611,0.7756,0.0107) is the one played.
% S is the total number of simulations in the Monte Carlo experiment.

% DecMat reports test conclusions: 1 for rejection and 0 for acceptance.
DecMat1 = []; DecMat2 = []; DecMat3 = [];

% DecSgn reports test conclusion: 1 for positive sign, -1 for negative
% sign and 0 for inconclusive.
DecSgn = [];

U = [.5,.3611,.3611]; % Delta_i = -1 for all i;
for s = 1:S, tic;
    % Step 1 : Simulate data.
    Ind = binornd(1,Lambda,G,1); Eps = normrnd(.1,.25,G,3);
    p1  = [.0611,.7756,.0107];  p2  = [.0611,.0107,.7756];
    D   = (Eps <= repmat(U,G,1)  ...
          - repmat(Ind,1,3).*repmat([p1(2)+p1(3),p1(1)+p1(3),p1(1)+p1(2)],G,1) ...
          - repmat(1-Ind,1,3).*repmat([p2(2)+p2(3),p2(1)+p2(3),p2(1)+p2(2)],G,1) ); 
    [T,Var,Stdev] = CalT(D);
    % Step 2 : Define the statistics for the sign test.
    A = sqrt(G)*T./Stdev;
    A = A';
    A = (A>1.64)-(A<-1.64);
    DecSgn = [DecSgn; A];
    % Now get the test-statistic for the remaining tests
    [ST, I] = sort(abs(T),1,'descend');
    % Step 3: Step-down procedure with simulated critical values (non-studentized, parametric)
    A = zeros(1,3);
    for i   = 1:length(I),
        % First find out the critical values. 
        CV  = FindCV(I(i:end),Var./G,1-alpha,B);
        % Then make a decision in the current step.
        if ST(i) > CV, A(1,I(i)) = 1; continue;
        else break;
        end;
    end;
    DecMat1 = [DecMat1; A]; 
    % Step 4: Romano & Wolf (2005) (non-studentized).
    A = zeros(1,3); 
    for i = 1:length(I),
        CV0 = NP_FindCV(I(i:end),D,1-alpha,B,T,0);
        if ST(i) > CV0, A(1,I(i)) = 1; continue;
        else break;
        end;
    end;
    DecMat2 = [DecMat2; A];
    % Step 5: Romano & Wolf (2005) (Studentized).
    [SST, I] = sort(abs(T)./diag(Var./G).^.5,1,'descend');
     A = zeros(1,3);
    for i = 1:length(I)
        CV1 = NP_FindCV(I(i:end),D,1-alpha,B,T,1);
        if SST(i) > CV1, A(1,I(i)) = 1; continue;
        else break;
        end;
    end;    
    DecMat3 = [DecMat3; A]; 
    toc;
end;
save(sprintf('MCP1_%d.mat',jobnum));