% This m-script performs Monte Carlo simulations for tests of signs of 
% interaction effects (Table 4) in de Paula and Tang (2011).  The 
% computations are stored in the workspace
% "MCP2_jobnum.mat" where jobnum is an argument in the function MCP2.

% When the workspace is opened in Matlab, the results are obtained from
% the matrices StatMat1, StatMat2 and StatMat3. These matrices are
% S by 2.  Each matrix corresponds to a player.  A row corresponds to a 
% Monte Carlo simulation and each column corresponds to a value in the 
% support of Xi.
% Table 4: The rows for Xi (where i is 1, 2 or 3) are obtained from StatMati.
% mean((StatMati>1.64)) is a vector containing q+ for each value in the 
% support of Xi.  mean((StatMati<-1.64)) is a vector of q- for each value 
% in the support of Xi.  

function MCP2(G,S2,delta,jobnum)

% G is the sample size in each simulation.
% S2 is the total number of simulations in the Monte Carlo experiment.
% delta is the scale parameter of the interaction effects.

% S specifies support of regressors.
S = {[1], [-1 2], [-1/2 3/2], [-1 3]}; 

% CalEQ3 calculates equilibrium for any given state
[EQ,flag]  = CalEQ3(delta,S); if flag == 1, disp('warning!'); return, end 
StatMat1 = [];
StatMat2 = [];
StatMat3 = [];

% begin the iteration in simulations
for s = 1:S2
    tic;
    % Generate the game data.
    % First generate the regressors and the errors.
    X = zeros(G,4); for i = 1:4, X(:,i) = randsample(S{i},G,true); end
    Eps = rand(G,3); 
    Eps = Eps.*4.*(abs(repmat(X(:,1),1,3).*X(:,2:4)) + repmat(X(:,1),1,3) + X(:,2:4))...
              -2.*(abs(repmat(X(:,1),1,3).*X(:,2:4)) + repmat(X(:,1),1,3) + X(:,2:4));
    % Next, match the vector of regressors with their equilibrium.
    P = zeros(G,3); IndArrayX ={}; 
    for i = 1:size(EQ,1),
        Ind = find(prod(double((kron(ones(G,1),EQ(i,1:4)) == X(:,1:4))),2)); 
        P(Ind,:) = kron(ones(length(Ind),1),EQ(i,5:7)); 
        IndArrayX = {IndArrayX{:},Ind'};
    end
    % Calculate the players' decisions in data according to equilibrium.
    D = double( (  Eps <= kron(X(:,1),ones(1,3))+X(:,2:4)+delta.*kron(X(:,1),ones(1,3)).*X(:,2:4)...
                  .*(kron(sum(P,2),ones(1,3))-P)  ) );
    Data = [X D P];
    % The following step is optional: double-check whether data is consistent
    % with equilibrium prediction.
    DEQ = zeros(size(EQ,1),size(EQ,2)); 
    for i = 1:size(EQ,1),
        DEQ(i,1:4) = EQ(i,1:4); DEQ(i,5:7) = mean(D(IndArrayX{i}',:));  
    end
    % We want the following difference to be close to zero:
    % double(EQ(:,5:7)) - DEQ(:,5:7).
    % Now we are ready to calculate the test statistics for this simulated sample
    % First we find subsets with given vectors of regressor values
    IndI = {[],[],[]};
    for i = 1:3, PreInd_ix = {};
        % EC_Xi consists of all possible combinations of X0 and Xi.
        EC_Xi   = [kron(S{1}',ones(length(S{i+1}),1)) kron(ones(length(S{1}),1),S{i+1}')];
        A = zeros(1,size(EC_Xi,1));
        for j   = 1:size(EC_Xi,1),
            % PreIndi contains indices of 
            pindi     = find(prod(double(kron(ones(size(EQ,1),1),EC_Xi(j,:)) == EQ(:,[1,i+1])),2)); 
            PreInd_ix = {PreInd_ix{:},[IndArrayX{pindi}]};
            Dsub      = D([IndArrayX{pindi}],:);
            % Calculate the test statistics
            sxi1 = size(Dsub,1)/G;        sxi2 = mean(Dsub(:,i).*(sum(Dsub,2)-Dsub(:,i)))*sxi1;
            sxi3 = mean(Dsub(:,i))*sxi1;  sxi4 = mean(sum(Dsub,2)-Dsub(:,i))*sxi1;
            sxi5 = mean(Dsub(:,i).*(sum(Dsub,2)-Dsub(:,i)).^2)*sxi1;   
            sxi6 = mean((sum(Dsub,2)-Dsub(:,i)).^2)*sxi1;
            v    = [-sxi2*sxi1^(-2)+2*sxi3*sxi4*sxi1^(-3), 1/sxi1, -sxi4/sxi1^2, -sxi3/sxi1^2];
            var  = [sxi1*(1-sxi1) sxi2*(1-sxi1)   sxi3*(1-sxi1)   sxi4*(1-sxi1);   ...
                    sxi2*(1-sxi1) sxi5-sxi2^2     sxi2*(1-sxi3)   sxi5-sxi2*sxi4; ...
                    sxi3*(1-sxi1) sxi2*(1-sxi3)   sxi3*(1-sxi3)   sxi2-sxi3*sxi4;   ...
                    sxi4*(1-sxi1) sxi5-sxi2*sxi4  sxi2-sxi3*sxi4  sxi6-sxi4^2     ];
            A(1,j) = G^(.5)*(sxi2/sxi1-sxi3*sxi4/sxi1^2)*(v*var*v')^(-.5); 
        end
        if i == 1 
            StatMat1 = [StatMat1; A]; 
        elseif i == 2 
            StatMat2 = [StatMat2; A]; 
        else
            StatMat3 = [StatMat3; A]; 
        end
        % This is just to keep track of which rows in the data correspond
        % to which vectors of regressors.
        IndI{i}  = PreInd_ix;
    end
    toc;
    
end

save(sprintf('MCP2_%d.mat',jobnum));

