//=======/=========/=========/=========/=========/=========/=========/========/=========/=========/=========/=========/=========/========/========/========//
//
// 	de Paula and Tang: Radio Data Illustration
//	(06/01/2011)
//
//  We use the command -egen xtile()- which is user-written and part of the -egenmore- package on SSC.
//
//=======/=========/=========/=========/=========/=========/=========/========/=========/=========/=========/=========/=========/========/========/========//

clear
clear matrix
clear mata

log using depaulatang.smcl, replace

set memory 500m
set more off

use radio.dta

// Defining cutoff values (number of firms, joint market share, number of bootstrap samples, FWER).
local numstations = 3  		// numstations: number of stations per market (between 1 and 9).
local minmktshare = 0.1  	// minmktshare: minimum joint market share.
local numboot = 1000 		// numboot: number of bootstrap samples.
local alpha = 97.5			// alpha: 100-FWER/2 (0<alpha<100).  FWER = 200-2*alpha.

//=======/=========/=========/=========/=========/=========/=========/========/=========/=========/=========/=========/=========/========/========/========//
// Compute the (individual) test statistics for ( 1 ) pooled data and ( 2 ) by hour.

// Program `myboot' to be used in the bootstrap for the Romano and Wolf (2005).
// The arguments for the program `myboot' are `choice5055' and `numstations'

cap program drop myboot
program define myboot
	preserve
	bsample
	egen totalchoice = rowtotal(`1'?)
	forvalues i = 1/`2' {
		gen gamma`i' = totalchoice-`1'`i'
	}
	qui corr `1'? gamma?, covariance	
	matrix COV = r(C)
	forvalues i = 1/`2' {
		scalar teststat`i' = COV[`2'+`i',`i']*(r(N)-1)/r(N)
	}
	restore
end

foreach dataset in "_all" "if hour==12" "if hour==16" "if hour==17" "if hour==21" {

	di _n "//=======/=========/=========/=========/=========/=========/=========/========/========//" _n ///
		"Results for `dataset'" _n

	forvalues l = 0/3 {

		qui keep `dataset'
		if `l'>0 {
			if "`dataset'" == "_all" {
				qui keep if (mktsizepool == `l')				
			}
			else {
				qui keep if (mktsizeh == `l')
			}
			di _n "mktsize = " `l' "." _n
		}
		
		foreach name in "choice5055" {
			di _n "Results for `name'" _n
			
			forvalue i = 1/`numstations' {
				drop if `name'`i' == .
			}

// Computes the Wald statistic for the simultaneous test.
			
			mata: namemata = st_local("name")
			mata: numstationsmata = strtoreal(st_local("numstations"))
			mata: st_matrix("wtstat",J(numstationsmata,1,.))
			egen totalchoice = rowtotal(`name'?)  		// Adds the choice variables across players.
			forvalues i = 1/`numstations' {
				gen gamma`i' = totalchoice-`name'`i'	// For each player, subtracts his/her choice from "totalchoice".
			}
			qui corr `name'? gamma?, covariance			// Generates covariance matrix between choice and gamma variables.
			matrix COV = r(C)							// Stores the matrix in COV.
			display "  number of obs = " r(N)
			forvalues i = 1/`numstations' {
				matrix wtstat[`i',1] =  COV[`numstations'+`i',`i']*(r(N)-1)/r(N)
			}											// The test statistic is the covariance b/w choice and gamma for each player.
			drop totalchoice gamma?
			mata: mu = st_data(.,namemata+"?")			// Forms a G x numstations matrix where each column corresponds to the choice by a player.
			mata: rows(mu)
			mata: for(i=1; i<=numstationsmata-1; i++) {; for(j=i+1; j<=numstationsmata; j++) {; addmu = mu[.,i] :* mu[.,j]; mu = mu, addmu; }; }
														// Expands the matrix to include cross-products of choice across different players.
			mata: meanmu = mean(mu)						// Calculates the means of the columns in mu.
			mata: mu = mu :- meanmu						// De-means mu.
			mata: sigma = cross(mu,mu):/rows(mu)		// Estimates the matrix sigma in the paper.
		
			mata: V = J(numstationsmata,numstationsmata,0)
			mata: for(i=1; i<=numstationsmata; i++) {; V[i,i]=meanmu[1,i]-sum(meanmu[1,1::numstationsmata]); }
			mata: for(i=1; i<=numstationsmata; i++) {; for(j=1; j<=numstationsmata; j++) {; if(i!=j) {; V[i,j]=-meanmu[1,i]; }; }; }
			mata: for(i=1; i<=numstationsmata-1; i++) {; for(j=i+1; j<=numstationsmata; j++) {; VV = J(numstationsmata,1,0); VV[i,1]=1; VV[j,1]=1; V=V, VV; }; }
		
			mata: st_matrix("invavar",rows(mu):*invsym(V*sigma*V')) 
														// Computes G * inverse of asymptotic variance estimate and transfers it to Stata
		
			matrix waldtstat`name'=wtstat'*invavar*wtstat
														// Computes the Wald statistic.
			display "  the Wald test statistic for `name' is " waldtstat`name'[1,1]
		
// Computes the Romano and Wolf (2005) procedure.
						
			egen totalchoice = rowtotal(`name'?)  		// Adds the choice variables across players.
			forvalues i = 1/`numstations' {
				gen gamma`i' = totalchoice-`name'`i'	// For each player, subtracts his/her choice from "totalchoice".
			}
			qui corr `name'? gamma?, covariance			// Generates covariance matrix between choice and gamma variables.
			matrix COV = r(C)							// Stores the matrix in COV.
			display "  number of obs = " r(N)
			forvalues i = 1/`numstations' {
				scalar teststat`i' =  COV[`numstations'+`i',`i']*(r(N)-1)/r(N)
				display "  teststat`i' = " teststat`i'
			}											// The test statistic is the covariance b/w choice and gamma for each player.
														// The adjustment *(r(N)-1)/r(N) is because the command generates 'sample' covariances, not
														// 'population' ones.

// Generates data files for each specification with the test statistics for use in bootstrap computation of critical values.
			drop _all
			set obs `numboot'
			forvalues i  = 1/`numstations' {
				gen tstat`i' = teststat`i'
			}
			qui save bootstrapfile`name', replace
			
			clear
			use radio
			qui keep `dataset'
			if `l'>0 {
				if "`dataset'"=="_all" {
					qui keep if (mktsizepool == `l')
				}
				else {
					qui keep if (mktsizeh == `l')
				}
			}
			
// Perform multiple comparisons procedure.
												  
// Calculate critical values via bootstrap.

// Generates a file for each one of "choice5055" with the `numstations' bootstrapped statistics for each bootstrapped sample.
		foreach name in "choice5055" {
			di _n "Bootstraping test statistics for `name'"	

			forvalues j = 1/`numstations' {
				qui simulate teststat`j', reps(`numboot') seed(12345): myboot `name' `numstations'
				// bstat, stat(teststat`j') notable
				
				gen teststat`j' = _sim_1
				drop _sim_1
				
				qui merge 1:1 _n using bootstrapfile`name'
				drop _merge
				qui save bootstrapfile`name', replace	
				
				clear
				use radio
				qui keep `dataset'
				if `l'>0 {
					if "`dataset'"=="_all" {
						qui keep if (mktsizepool == `l')
					}
					else {
						qui keep if (mktsizeh == `l')
					}
				}
			}
		}

// Perform the step-down procedure, eliminating the variable with the highest test statistic at each step and recomputing the maximum of the bootstrapped
// test statistics in each step.
		foreach name in "choice5055" {
			di _n "Test results for `name'" _n
			clear
			use bootstrapfile`name'
			forvalues i = 1/`numstations' {
				gen btstat`i'=abs(teststat`i'-tstat`i')
				gen abststat`i'=abs(tstat`i')
				drop teststat`i'
			}
			forvalues i = 1/`numstations' {
				local perc = int(100*(`numstations'-`i'+1)/(`numstations'+1))
				egen maxtstat`i' = rowpctile(abststat?), p(`perc')			// Orders the test statistics in decreasing order.
			}
			forvalues i = 1/`numstations' {
				forvalues j = 1/`numstations' {
					if abststat`j'==maxtstat`i' {
						egen maxbtstat = rowmax(btstat?)
						_pctile maxbtstat, p(`alpha')
						if maxtstat`i'[1] < r(r1) {
							di "No null hypotheses are rejected for `name' in step `i'."
							continue, break
						}
						else {
							di "Failed to accept null for player `j'. The test statistic is " tstat`j' " and the critical level is +/- " r(r1) "."
							drop btstat`j'
							continue, break
						}
					}
				}
				drop maxbtstat
			}		
		}
		clear
		use radio	
	}
	}
}
log close
translate depaulatang.smcl depaulatang.log, replace
